"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_helpers_1 = require("@previewjs/ts-helpers");
const vfs_1 = require("@previewjs/vfs");
const path_1 = __importDefault(require("path"));
const extract_component_1 = require("./extract-component");
const MAIN_FILE = path_1.default.join(__dirname, "virtual", "App.tsx");
describe("extractReactComponents", () => {
    let memoryReader;
    let typeTracker;
    beforeEach(() => {
        memoryReader = (0, vfs_1.createMemoryReader)();
        typeTracker = new ts_helpers_1.TypeTracker({
            rootDirPath: path_1.default.join(__dirname, "virtual"),
            reader: (0, vfs_1.createStackedReader)([
                memoryReader,
                (0, vfs_1.createFileSystemReader)({
                    mapping: {
                        from: path_1.default.resolve(__dirname, "..", "..", "..", "node_modules"),
                        to: path_1.default.join(__dirname, "virtual", "node_modules"),
                    },
                }),
                (0, vfs_1.createFileSystemReader)(), // required for TypeScript libs, e.g. Promise
            ]),
            typeRoots: [],
        });
        typeTracker.start();
    });
    afterEach(() => {
        typeTracker.stop();
    });
    it("detects expected components", async () => {
        expect(extract(`
import React, { Component } from "react";

function wrapComponent<P>(c: React.ComponentType<P>): React.ComponentType<P> {
  return c;
}

function DeclaredFunction() {
  return <div>Hello, World!</div>;
}

const ConstantFunction = () => <div>Hello, World!</div>;

// Note: this isn't detected as of October 2021.
const HocComponent = wrapComponent(() => <div>Hello, World!</div>);

class ClassComponent1 extends React.Component {}

class ClassComponent2 extends Component {}

const ForwardRef = React.forwardRef((props, ref) => <ConstantFunction {...props} />);

export class BaseClass<T> {}

export class NotComponentClass extends BaseClass<SomeProps> {}

export const NotObjectProps = (props: () => void) => {
  return <div>Hello, World!</div>;
};

export const MissingType = (props) => {
  return <div>Hello, World!</div>;
};

export const NotComponent = () => {
  // This isn't a component.
};

export const NotComponentEither = () => {
  return "Hello";
};
`)).toEqual([
            {
                name: "DeclaredFunction",
                exported: false,
                offsets: expect.anything(),
            },
            {
                name: "ConstantFunction",
                exported: false,
                offsets: expect.anything(),
            },
            // Note: this isn't detected as of October 2021.
            // {
            //   name: "HocComponent",
            //   exported: false,
            //   offsets: expect.anything(),
            // },
            {
                name: "ClassComponent1",
                exported: false,
                offsets: expect.anything(),
            },
            {
                name: "ClassComponent2",
                exported: false,
                offsets: expect.anything(),
            },
            {
                name: "ForwardRef",
                exported: false,
                offsets: expect.anything(),
            },
            {
                name: "NotObjectProps",
                exported: true,
                offsets: expect.anything(),
            },
            {
                name: "MissingType",
                exported: true,
                offsets: expect.anything(),
            },
        ]);
    });
    function extract(source) {
        const rootDirPath = path_1.default.join(__dirname, "virtual");
        memoryReader.updateFile(path_1.default.join(rootDirPath, "App.tsx"), source);
        return (0, extract_component_1.extractReactComponents)(typeTracker.analyzeProgram([MAIN_FILE]), MAIN_FILE);
    }
});
