"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.reactFrameworkPlugin = void 0;
const path_1 = __importDefault(require("path"));
const component_loader_plugin_1 = require("./component-loader-plugin");
const extract_component_1 = require("./extract-component");
const optimize_deps_plugin_1 = require("./optimize-deps-plugin");
const react_imports_plugin_1 = require("./react-imports-plugin");
const svgr_plugin_1 = require("./svgr-plugin");
exports.reactFrameworkPlugin = {
    requirements: {
        packageName: "react",
        minMajorVersion: 16,
    },
    create: (options = {}) => {
        return {
            defaultWrapperPath: () => "__reactpreview__/Wrapper.tsx",
            previewDirPath: () => path_1.default.resolve(__dirname, "..", "preview"),
            componentDetector: (analyzeProgram, filePaths) => {
                let program;
                try {
                    program = analyzeProgram(filePaths);
                }
                catch (e) {
                    return () => [];
                }
                return (filePath) => (0, extract_component_1.extractReactComponents)(program, filePath);
            },
            viteConfig: (config) => {
                var _a;
                return ({
                    plugins: [
                        (0, optimize_deps_plugin_1.optimizeReactDepsPlugin)(),
                        (0, component_loader_plugin_1.reactComponentLoaderPlugin)({
                            config,
                        }),
                        (0, svgr_plugin_1.svgrPlugin)({
                            exportedComponentName: ((_a = options.svgr) === null || _a === void 0 ? void 0 : _a.componentName) || "ReactComponent",
                            alias: config.alias,
                        }),
                        (0, react_imports_plugin_1.reactImportsPlugin)(),
                    ],
                });
            },
        };
    },
};
