import { FrameworkPluginFactory } from "@previewjs/core";
import path from "path";
import ts from "typescript";
import { reactComponentLoaderPlugin } from "./component-loader-plugin";
import { extractReactComponents } from "./extract-component";
import { optimizeReactDepsPlugin } from "./optimize-deps-plugin";
import { reactImportsPlugin } from "./react-imports-plugin";
import { svgrPlugin } from "./svgr-plugin";

export const reactFrameworkPlugin: FrameworkPluginFactory = {
  requirements: {
    packageName: "react",
    minMajorVersion: 16,
  },
  create: (
    options: {
      svgr?: {
        componentName: string;
      };
    } = {}
  ) => {
    return {
      defaultWrapperPath: () => "__reactpreview__/Wrapper.tsx",
      previewDirPath: () => path.resolve(__dirname, "..", "preview"),
      componentDetector: (analyzeProgram, filePaths) => {
        let program: ts.Program;
        try {
          program = analyzeProgram(filePaths);
        } catch (e) {
          return () => [];
        }
        return (filePath: string) => extractReactComponents(program, filePath);
      },
      viteConfig: (config) => ({
        plugins: [
          optimizeReactDepsPlugin(),
          reactComponentLoaderPlugin({
            config,
          }),
          svgrPlugin({
            exportedComponentName:
              options.svgr?.componentName || "ReactComponent",
            alias: config.alias,
          }),
          reactImportsPlugin(),
        ],
      }),
    };
  },
};
